
pro first_lecture

;Lecture notes on the short introductory course in IDL that was organized and held in the
;Max Planck Institute for Astronomy - Heidelberg, Germany.
;=================================== 
;FIRST LECTURE 
;for further infos contact NICOLA DA RIO, dario@mpia.de
;===================================

print,'hello world'
PRINT,'hello world'   ;IDL is not case sentitive
print   ,    'hello world' ;additional spaces don't matter
print,$       
  'hello world'    ;you can continue a command on the next line with $
print,'hello' & print,'world'  ;you can call multiple commands in a line with &

;===================================
; Define variables
;===================================

x=1 ;int 16
y=1. ;float 32
help,x,y
print,x,y

z=2b ;byte 8
t=2l ;long 32
r=2LL ;long 64
p=2d ;float 64
help,z,t,r,p

a='hello world'
help,a
print,a

;===================================
;Environmental variables,
;===================================
print,!pi

print,!dpi
print,!dtor ; pi/180
print,!radeg ;180/pi

print,!values.f_infinity
print,!values.f_nan

;===================================
;Define arrays
;===================================

x=fltarr(10) ; array of zeros with 10 elements
print,x
y=intarr(3,2) ; array of zeros with 3 columns x 2 rows
print,y

;Alternatively, indgen, findgen, dindgen, etc.
x=findgen(10) ; index generator from 0 to 10-1
print,x
y=indgen(3,2) ; index generator from 0 to 3x2-1
print,y

z=double(y) ; change the type of a variable
print,z
print,floor(3.6),ceil(3.4),round(3.6)
t=[1,5,3] ; manually create arrays
print,t
z=[1000,[t]]   ; append arrays
print,z
t=[[1,2,3],[7,6,5]]
print,t


;===================================
;Work with elements of variables,
;===================================

x=findgen(10)+2.5
x[2]=99 ; change one element
print,x

x[5:8]=99 ; change more elements
print,x

x[5:*]=0
print,x

i=1
x[i]=i*3
print,x
;(4b)
;note that these two are not equivalent,
x[*]=3
print,x
x=3
print,x

a=findgen(10,20)
print,a
help,a
b=transpose(a)
print,b
help,b

;reverse by rows
c=reverse(a)
print,a[*,0]
print,c[*,0]
print,a[0,*]
print,c[0,*]

;reverse by colums
c=reverse(a,2)
print,a[*,0]
print,c[*,0]
print,a[0,*]
print,c[0,*]

;other useful functions is ROTATE

;the subscripts of nD arrays (n ge 2) can be either multiple,
;one for each dimension, or simple scalars, from 0 to n_elements-1
print,a[2,2] ;element of third row and third column
print,a[22]  ;22st element of the array, counting starting from first row, all the columns, second row, all the colums,..

result=a mod 3
multiples_of_3=where(result eq 0)
a[multiples_of_3]=!values.f_nan

c=where(finite(a) eq 1)

;===================================
; Operators, +,-,*,/,mod,^,#,##,
;===================================

x=findgen(10)
y=10-findgen(10)
print,x+y,x-y,x*y,x/y

z=x*3
print,x
z=y^2
print,z

;be careful with arrays of different size!!
;the dimention of the result is equal to the dimension of the smallest 
a=findgen(10)
b=findgen(5)
c=a+b
help,c
print,c

;also for individual subscripts
z=x[0:3]-y[6:9]
print,z

z=x[0:4]-y[1:9] ; this is also allowed, but careful with the result!
print,z

x=findgen(3,2) ; another (correct) example
y=findgen(5,3)

z=x[*,0]*y[2,*]
help,x[*,0],y[2,*]
print,z


; Dive into the operators,

x=fltarr(10)
;the following operations are in principle equivalent
x=x+2.5
x+=2.5

;but actually,
x=fltarr(10000000)
t0=systime(1) & x=x+2.5 & print,systime(1)-t0
x=fltarr(10000000)
t0=systime(1) & x+=2.5 & print,systime(1)-t0

;also the following operations seem to be doing the same
y=x^2
y=x*x

;operations between numbers (or arrays) of different type
a=1
b=1.
c=1L ;or c=long(1)
d=1D ;or d=double(1)
help,a+b
help,b+c
help,a*c
help,c-d
;the precision is always preserved
;but be careful with divisions between two integers


;useful functions 

x=findgen(10,10)

print, n_elements(x)
print, size(x)
print, total(x)
print, total(x,1),  total(x,2)
print, mean(x), median(x), stddev(x)
print,  median(x, dimension=1)


;===================================
;IF statements,
;===================================

;if (condition) then (statement)
;
;
;if (condition) then begin
;    (statement1)
;    (statement2)
;    ...
;endif
;
;
;if (condition1) then (statement1) else (statement2)
;
;
;if (condition) then begin
;    (statement1)
;    (statement2)
;    ...
;endif else begin
;    (statement34)
;    (statement35)
;    ...
;endelse


;example using eq, ne, ge, gt, le, lt

a=12
x=0
if a eq 0 then x=10 else print,x
if a ge 0 then x=10 else x=1000

;advanced example, and, or, xor,
if a gt 0 and x gt a then print, "true" else print, "false"
if a gt 0 or x gt a then print, "true" else print, "false"
if a gt 0 xor x gt a then print, "true" else print, "false"

;example of negation ~

print, (a gt 0), ~(a gt 0)

;===================================
;loops, for, while, repeat
;===================================

;for i=start,finish [,step] do operation

for i=0,5 do print, i

n=40
x=findgen(n)
y=findgen(n)
for i=0,n-1 do begin & $
    x[i]=i  & $
    y[i]=x[i]+i   & $
endfor

;example
x=-findgen(10)/2
y=fltarr(5)
for i=0,4 do y[i]=x[i*2]
print,x,y

j=indgen(5)*2 ; avoid such loops
y=x[j]
print,y

;nested loops
for i=1,7,2 do for k=12,15 do print, i,k

x=fltarr(4,3)
for i=0,3 do for k=0,2 do x[i,k]=1+i*k^2 ; <-- pay attention!
print,x


;starting variable
x=0.
for i=1000,100000 do x+=i  ;doesn't work: i is integer, lt 2^15-1
for i=1000L,100000 do x+=i ; <--- correct definition

x=0.
for i=0. ,10, .1 do x+=i ; <--- correct definition

;while
;
;while (condition) do operation
;
;while (condition) do begin
;    operation1
;    ...
;endwhile

j=0
while j le 10 do j+=1
print,j


;eternity loop?
;while (true) do statement

j=0
while 1 do if ++j eq 10 then break
print,j


;break, continue statements
for i=0,10 do begin & $
    if i eq 2 then continue & $ ; skips one loop if true
    if i eq 5 then break   & $  ; interrupts the loop if true
    print,i & $
    ;(some operations)
    ;...
endfor


;CASE statement
x=2 
CASE x OF & $
   1: PRINT, 'one' & $
   2: PRINT, 'two' & $
   3: PRINT, 'three'  & $
   4: PRINT, 'four' & $
ENDCASE 


;===================================
;Read and write ascii files,
;===================================

;to write an ascii table in a file use
openw,1,'some_data.tab'
for i=0.,3,0.3 do printf,1,i,i*2
close,1

openw,1,'some_data.tab',/append
for i=0.,3,0.3 do printf,1,i,i*2
close,1

;but you can use the forprint routine
;remember to run astrolib first
astrolib
forprint,y,y^0.49,textout='some_data.tab'


;to read an ascii table you can use
openr,1,'some_data.tab'
;and then 
readf,1,a,b
close,1

;but to use readcol is even smarter
readcol,'some_data.tab',x,y
print,x,y


;===================================
; plotting
;===================================

cd,'C:\Documents and Settings\Nicola\Documenti\_materiale_ricerca\IDL_course'
fileout='ascii1.dat'
readcol,fileout,col1,col2,col3

window              ;open a window
plot,col1
plot,col1,col2      ;plotting
window,1       ;open another window, with unit 1
window,/free,title='this is another windows'   ;open a free window

wdelete,1           ;closes the window number 1
window,1,xsize=512,ysize=512
plot,col1,col2
plot,col1,col2,xrange=[20,29],yrange=[0,1.2]   ;setting limits
plot,col1,col2,xrange=[20,29],yrange=[0,1.2],xstyle=1  ;strong constraint on limits

plot,col1,col2,xrange=[20,29],yrange=[0,1.2],xstyle=1,psym=4   ;psym=0 is a line, psym from 1 to 8 are symbols
plot,col1,col2,xrange=[20,29],yrange=[0,1.2],xstyle=1,psym=4,symsize=2.3
plot,col1,col2,xrange=[20,29],yrange=[0,1.2],xstyle=1,psym=4,symsize=2.3,thick=2
plot,col1,col2,psym=0,thick=2,linestyle=1

usersym,[1,1,0,-1,-1,1],[0,1,2,1,0,0],/fill   ;defines the 8th symbol type (to be used called psym=8, using vertices of points)
plot,col1,col2,xrange=[20,29],yrange=[0,1.2],xstyle=1,psym=8,symsize=2.3,thick=2,linestyle=1
plotsym,0,/fill    ;this is a useful procedure included in the ASTROLIB, to quick define circles and arrows
plot,col1,col2,xrange=[20,29],yrange=[0,1.2],xstyle=1,psym=8,symsize=1,thick=2,linestyle=1,xtitle='this is X',ytitle='this is Y',title='this is a plot',xcharsize=1.5,ycharsize=1.5

oplot,col1,col2*1.15    ;overplot something else
loadct,5            ;fifth color palette, called 'STD GAMMA-II'
oplot,col1,col2*1.15,color=128  ;overplotting using the 128th color of the loaded color table

plots,21.0,0.8,psym=8             ;overplots a point or a series defined by x and y arrays
plots,[20.5,21],[0.7,0.7],color=128
xyouts,21.2,0.8,'my data',charsize=1.2  ;xyouts procedure puts a label in the specified position
xyouts,21.2,0.7,'something else',charsize=1.2


xloadct           ;enables to modify manually the color tables

R=indgen(256)
G=reverse(indgen(256))
B=replicate(128,256)
G[0]=0
B[0]=0
modifyct,42,'my color table',R,G,B ;modify the 42nd color table
loadct,42                ;loads it
plot,[0,1],[0,0],yrange=[0,20]
for i=0,255 do oplot,[0,1],[i/10.,i/10.],color=i

;plotting with error bars
ecol1=replicate(0.2,n_elements(col1))
ecol2=(1.1-col2)*0.1
loadct,0
ploterror,col1,col2,ecol1,ecol2   ;this is part of the ASTROLIBs
ploterror,col1,col2,ecol2,hat=0

;interaction with the cursor
print,'please, click on the window'
cursor,xc,yc,/data
print,xc,yc
col2b=col2
col2b[where(col1 le xc)]=0
oplot,col1,col2b

;doing different things when you left or right click
loadct,5
for k=0,5 do begin & $
  cursor,xc,yc,/data & $
  if (!mouse.button eq 1) then begin & $
      print,'you have done a left click' & $
      plots,xc,yc,/data,psym=4,color=60,thick=2,symsize=4 & $
  endif & $
  if (!mouse.button eq 4) then begin & $
      print,'you have done a right click' & $
      plots,xc,yc,/data,psym=4,color=120,thick=2,symsize=4 & $
  endif & $
  wait,0.2 & $
endfor

;=====================================
; PLOTTING TO POSTSCRIPT FILE
;=====================================

fileout='plot1.ps'
set_plot,'ps'
device,bits=8,filename=fileout,/portrait,/color,xsize=16.,ysize=12.,yoffset=3.
;device,bits=8,filename=fileout,/landscape

;here put whatever you want to plot
plot,col1,col2,psym=8

;closing the file
device,/close
set_plot,'win'    ;holds on win systems
device,decomposed=0.

;=====================================
;MULTIPLE PLOTS
;=====================================

;in order to have multiple plots on the same window, change the system variable !p.multi

window,1
!p.multi=0   ;means one plot per page/window. It is the default setting
plot,sqrt(findgen(100))

!p.multi=[0,3,2]   ;the second element of the declared array is the number of plots in X
             ;the third is the number of plots in Y
plot,sqrt(findgen(100))
plot,sqrt(findgen(50))*2
plot,(findgen(100))
plot,(findgen(100))^2
plot,sin(findgen(100)/10)
plot,cos(findgen(100)/10),color=128,title='last plot'

!p.multi=0

;=====================================
;MULTIPLE PLOTS with SHARED AXES
;=====================================
;use the ASTROLIB function 'multiplot'
erase & multiplot,[2,2],mXtitle='R', mYtitle='F(R)'
plot,sqrt(findgen(100)),xrange=[0,100],yrange=[0,12]
multiplot
plot,sqrt(findgen(100))+2,xrange=[0,100],yrange=[0,12]
multiplot
plot,sqrt(findgen(100))+4,xrange=[0,100],yrange=[0,12]
multiplot
plot,sqrt(findgen(100))+6,xrange=[0,100],yrange=[0,12]
multiplot
multiplot,/reset

;======================================
; FITS FILE OPENING
;======================================

filename='image1.fits'

;the procedure readfits is included in the astrolib.
;it reads the fits file defined by filename, and stores
;the image into the 2D array 'image' and the header into the variable header
image=readfits(filename,header)
help,image
;or, instead, I can use the function size
mysize=size(image)
help, mysize  ;it is a vector
print, mysize[0], 'this is the number of dimensions of the array'
print, mysize[1], 'this is the size of the first dimension'
print, mysize[2], 'this is the size of the second dimension'

;======================================
;image displaying
;======================================
window,1,xsize=mysize[1],ysize=mysize[2]
loadct,0   ;black/white color table
tvscl,image
print,'minimum value = ',min(image)
print,'maximum value = ',max(image)

image2=image>0<800   ;image will be equal to image1 except where it was less than 0, which are defined now equal 0
              ;and the points greater than 10, now defined equal 10
tvscl,image2

tvscl,sigrange(image)

;======================================
;histograms
;======================================

window,2
plothist,image   ;draws the histogram of the values of the array 'image'
plothist,image,/ylog,bin=5,xrange=[0,2000],xstyle=1 ;the keyword /ylog is valid for most of the plot procedure
                                                             ;the keyword 'bin' forces a manual bin size for the histogram
;======================================
;finding absolute maximum
;======================================

window,1,xsize=mysize[1],ysize=mysize[2],xpos=400
loadct,5
tvscl,sigrange(image>0)
maxvalue=max(image)
maxpoint=where(image eq maxvalue)
print,maxpoint
maxpointx=maxpoint mod mysize[1]
maxpointy=maxpoint / mysize[1]
tvcircle,100,maxpointx,maxpointy

image2=image
image2[where(shift(dist(mysize[1],mysize[2]),maxpointx,maxpointy) ge 40)]=0
tvscl,image2<800>0

;refining the centroid using the cntrd procedure (astrolib)
;Syntax: CNTRD, img, x, y, xcen, ycen, [ fwhm,
;   EXTENDBOX= , /KEEPCENTER, /SILENT, /DEBUG ]
cntrd,image,maxpointx,maxpointy,newx,newy,10
print,newx-maxpointx
print,newy-maxpointy

;======================================
; dealing with header keywords
;======================================

for i=0,n_elements(header)-1 do print,header[i]
exptime=sxpar(header,'exptime') ;extracts the value of the keyword 'exptime' into a variable
airmass=sxpar(header,'airmass')
help,exptime,airmass

norm_image=image/exptime

norm_header=header
fxaddpar,norm_header,'exptime',1.,'new exposure time'   ;modifies the exptime keyword, putting value 1 and adding comment
fxaddpar,norm_header,'AUTHOR','Nicola Da Rio','it is me' ;adding another keyword


;======================================
; writing fits images
;======================================

fileout=DATAPATH+'imageout.fits'
writefits,fileout,norm_image,norm_header     ;write the normalized image into fileout, including the modified header

writefits,fileout,norm_image          ; if you don't declare an header, a minimal header will be created
im=readfits(fileout,hdr)
for i=0,n_elements(hdr)-1 do print,hdr[i]
delvarx,im    ;deallocating variable im

;======================================
;getting RA and DEC of the point
;======================================
xyad,header,newx,newy,ra,dec   ;converts the image coordinates (pixels) into ra and dec using header's WCS
print,ra,dec
adxy,header,84.2,-66,xx,yy
print,xx,yy

;======================================
;extracting subimage
;======================================

stamp=image[(newx-40):(newx+40),(newy-40):(newy+40)]
help,stamp
window,1,xsize=(size(stamp))[1],ysize=(size(stamp))[2]
tvscl,stamp>0<800

;======================================
;resampling image
;======================================

bigstamp1=congrid(stamp,(size(stamp))[1]*4,(size(stamp))[2]*4)          ;4times each dimension, no interpolation
window,2,xsize=(size(bigstamp1))[1],ysize=(size(bigstamp1))[2]
tvscl,bigstamp1>0<800
bigstamp2=congrid(stamp,(size(stamp))[1]*4,(size(stamp))[2]*4,/interp)  ;4times each dimension, interpolation
tvscl,bigstamp2>0<800
tvscl,bigstamp2-bigstamp1     ;residuals after interpolation

;======================================
;smoothing
;======================================

bigstamp3=smooth(bigstamp2,50)  ;smoothing with boxcar method, size=50pixels
tvscl,bigstamp3>0<800
bigstamp3=smooth(bigstamp2,50,/edge_truncate)
tvscl,bigstamp3>0<800

;======================================
;contour plots
;======================================

contour,bigstamp3,xrange=[0,(size(bigstamp3))[1]],yrange=[0,(size(bigstamp3))[2]], $
       xstyle=1,ystyle=1

mylevels=[200,300,400,500,600,800,1000,1400]
contour,bigstamp3,xrange=[0,(size(bigstamp3))[1]],yrange=[0,(size(bigstamp3))[2]], $
       xstyle=1,ystyle=1,levels=mylevels,/fill
contour,bigstamp3,xrange=[0,(size(bigstamp3))[1]],yrange=[0,(size(bigstamp3))[2]], $
       xstyle=1,ystyle=1,levels=mylevels,/overplot,c_annotation=strtrim(string(mylevels,2)), $
       c_charsize=1.5,c_charthick=2

;======================================
;plotting surfaces
;======================================

surface,bigstamp3
shade_surf,bigstamp3
shade_surf,bigstamp3,/zlog,min_value=0.01

;======================================
;interpolation
;======================================

readcol,'ascii1.dat',x,y
plot,x,y
oplot,x,y,psym=4

xx=dindgen(1000)/100.+20
;linear interpolation
yylinear=interpol(y,x,xx)  ;x and y are the original points, xx the new abscissa
oplot,xx,yylinear,psym=4,color=128

;spline
yyspline=spline(x,y,xx)    ;cubic spline. WARNING: different order of the parameters!
oplot,xx,yyspline,color=40,thick=2

y[where(y eq max(y))]=max(y)/2.
yylinear=interpol(y,x,xx)  ;x and y are the original points, xx the new abscissa
oplot,xx,yylinear,psym=4,color=128
yyspline=spline(x,y,xx)    ;cubic spline. WARNING: different order of the parameters!
oplot,xx,yyspline,color=40,thick=2

;======================================
;basic fitting
;======================================

readcol,'ascii1.dat',x,y

xx=dindgen(1000)/100.+20
;polinomial
c=poly_fit(x,y,4,/double)      ;polinomial fitting of 4th order
help,c
yy=c[0]+(c[1]*xx)+(c[2]*xx^2)+(c[3]*xx^3)+(c[4]*xx^4)
plot,x,y
oplot,xx,yy,color=128,thick=2

;linear,with errors
trimx=x[where(x ge 26 and x le 28)]
trimy=y[where(x ge 26 and x le 28)]
etrimy=(1.2-sqrt(trimy))/5
ploterror,trimx,trimy,etrimy
c1=linfit(trimx,trimy)
c2=linfit(trimx,trimy,measure_errors=etrimy)

y1=c1[0]+c1[1]*trimx
y2=c2[0]+c2[1]*trimx
oplot,trimx,y1,linestyle=1
oplot,trimx,y2,linestyle=2

;robust
trimy[1]=0.1

plot,trimx,trimy,psym=4,yrange=[0,1.2]
c1=linfit(trimx,trimy)
y1=c1[0]+c1[1]*trimx
c2=robust_linefit(trimx,trimy,y2)    ;the robust_linefit procedure is part of the astrolibs
oplot,trimx,y1,linestyle=1
oplot,trimx,y2,linestyle=2




end

pro helloworld,delimiter,c,print=print,secondword=secondword
a='hello'
if keyword_set(secondword) then b=secondword else b='world'
if keyword_set(delimiter) then begin
  c=a+delimiter+b
endif else begin
  c=a+' '+b
endelse
if keyword_set(print) then print,c
end

function addone,in
out=in+1
return,out
end